import { NodeType, } from './types/types';
import { toSingularAction } from './singular';
import { nodeToType } from './structure';
// ============== single ==============
export const single_actionTypes = (nodeName) => {
    return {
        get: `GET_${nodeName.toUpperCase()}`,
        setAll: `SET_${nodeName.toUpperCase()}`,
        patch: `PATCH_${nodeName.toUpperCase()}`,
    };
};
// ============== queue ==============
export const queue_actionTypes = (nodeName) => {
    return {
        get: `GET_${nodeName.toUpperCase()}`,
        setAll: `SET_${nodeName.toUpperCase()}`,
        push: `PUSH_${toSingularAction(nodeName)}`,
        pop: `POP_${toSingularAction(nodeName)}`,
        clear: `CLEAR_${nodeName.toUpperCase()}`,
    };
};
// ============== collection ==============
export const collection_actionTypes = (nodeName) => {
    return {
        get: `GET_${nodeName.toUpperCase()}`,
        setAll: `SET_${nodeName.toUpperCase()}`,
        set: `SET_${toSingularAction(nodeName)}`,
        add: `ADD_${toSingularAction(nodeName)}`,
        patch: `PATCH_${toSingularAction(nodeName)}`,
        delete: `DELETE_${toSingularAction(nodeName)}`,
    };
};
// ============== grouped list ==============
export const groupedList_actionTypes = (nodeName) => {
    return {
        get: `GET_${nodeName.toUpperCase()}`,
        setAll: `SET_${nodeName.toUpperCase()}`,
        set: `SET_${toSingularAction(nodeName)}`,
        add: `ADD_${toSingularAction(nodeName)}`,
        patch: `PATCH_${toSingularAction(nodeName)}`,
        delete: `DELETE_${toSingularAction(nodeName)}`,
        getItems: `GET_${nodeName.toUpperCase()}_ITEMS`,
        setItems: `SET_${nodeName.toUpperCase()}_ITEMS`,
        pushItem: `PUSH_${nodeName.toUpperCase()}_ITEM`,
        popItem: `POP_${nodeName.toUpperCase()}_ITEM`,
        deleteItem: `DELETE_${nodeName.toUpperCase()}_ITEM`,
        patchItem: `PATCH_${nodeName.toUpperCase()}_ITEM`,
        clearItems: `CLEAR_${nodeName.toUpperCase()}_ITEMS`,
        pushManyItems: `PUSH_MANY_${nodeName.toUpperCase()}_ITEMS`,
    };
};
// ============== from store structure ==============
export const generateActionTypesForStore = (storeState) => {
    let output = {};
    const keys = Object.keys(storeState);
    for (const key of keys) {
        const value = storeState[key];
        const type = nodeToType(value);
        switch (type) {
            case NodeType.SINGLE_NODE:
                output[key] = single_actionTypes(String(key));
                break;
            case NodeType.QUEUE_NODE:
                output[key] = queue_actionTypes(String(key));
                break;
            case NodeType.COLLECTION_NODE:
                output[key] = collection_actionTypes(String(key));
                break;
            case NodeType.GROUPED_LIST_NODE:
                output[key] = groupedList_actionTypes(String(key));
                break;
        }
    }
    return output;
};
const GET_VERBS = ['get'];
const LOCAL_VERBS = [
    'setAll',
    'set',
    'setMany',
    'pushMany',
    'setItems',
    'pushManyItems',
];
export const generateActionTypesDictionaryForStore = (storeState) => {
    let output = {};
    const storeActionTypes = generateActionTypesForStore(storeState);
    const nodeKeys = Object.keys(storeActionTypes);
    for (const nodeName of nodeKeys) {
        const bag = storeActionTypes[nodeName];
        Object.keys(bag).forEach((apiVerb) => {
            const actionType = bag[apiVerb];
            output[actionType] = {
                id: actionType,
                verb: apiVerb,
                nodeName,
            };
            if (LOCAL_VERBS.includes(apiVerb)) {
                output[actionType].isLocal = true;
            }
            if (GET_VERBS.includes(apiVerb)) {
                output[actionType].isGet = true;
            }
        });
    }
    return output;
};
