import { NodeType, } from './types/types';
import { toSingularAction } from './singular';
import { nodeToType } from './structure';
export const SIGNATURE = {
    '@@redux-store-generator/AUTO_GENERATED_ACTION': true,
};
// ============== single ==============
export const single_patchAction = (nodeName, extra) => (payload) => {
    return {
        type: `PATCH_${nodeName.toUpperCase()}`,
        payload,
        ...extra,
    };
};
export const single_setAction = (nodeName, extra) => (payload) => {
    return {
        type: `SET_${nodeName.toUpperCase()}`,
        payload,
        ...extra,
    };
};
export const single_getAction = (nodeName, extra) => () => {
    return {
        type: `GET_${nodeName.toUpperCase()}`,
        ...extra,
    };
};
export const single_all = (nodeName, extra) => {
    return {
        get: single_getAction(nodeName, extra),
        setAll: single_setAction(nodeName, extra),
        patch: single_patchAction(nodeName, extra),
    };
};
// ============== queue ==============
export const queue_getAction = (nodeName, extra) => (payload = {}) => {
    return {
        type: `GET_${nodeName.toUpperCase()}`,
        payload,
        ...extra,
    };
};
export const queue_setAction = (nodeName, extra) => (payload) => {
    return {
        type: `SET_${nodeName.toUpperCase()}`,
        payload,
        ...extra,
    };
};
export const queue_pushAction = (nodeName, extra) => (payload) => {
    return {
        type: `PUSH_${toSingularAction(nodeName)}`,
        payload,
        ...extra,
    };
};
export const queue_pushManyAction = (nodeName, extra) => (payload) => {
    return {
        type: `PUSH_MANY_${nodeName.toUpperCase()}`,
        payload: {
            items: payload,
        },
        ...extra,
    };
};
export const queue_popAction = (nodeName, extra) => () => {
    return {
        type: `POP_${toSingularAction(nodeName)}`,
        ...extra,
    };
};
export const queue_clearAction = (nodeName, extra) => () => {
    return {
        type: `CLEAR_${nodeName.toUpperCase()}`,
        ...extra,
    };
};
export const queue_all = (nodeName, extra) => {
    return {
        get: queue_getAction(nodeName, extra),
        setAll: queue_setAction(nodeName, extra),
        push: queue_pushAction(nodeName, extra),
        pop: queue_popAction(nodeName, extra),
        clear: queue_clearAction(nodeName, extra),
        pushMany: queue_pushManyAction(nodeName, extra),
    };
};
// ============== collection ==============
export const collection_getAction = (nodeName, extra) => (payload = {}) => {
    return {
        type: `GET_${nodeName.toUpperCase()}`,
        payload,
        ...extra,
    };
};
export const collection_setAllAction = (nodeName, extra) => (payload) => {
    return {
        type: `SET_${nodeName.toUpperCase()}`,
        payload,
        ...extra,
    };
};
export const collection_addAction = (nodeName, extra) => (payload) => {
    return {
        type: `ADD_${toSingularAction(nodeName)}`,
        payload: {
            ...payload,
        },
        ...extra,
    };
};
export const collection_setAction = (nodeName, extra) => (id, payload) => {
    return {
        type: `SET_${toSingularAction(nodeName)}`,
        id,
        payload: {
            ...payload,
        },
        ...extra,
    };
};
export const collection_patchAction = (nodeName, extra) => (id, payload) => {
    return {
        type: `PATCH_${toSingularAction(nodeName)}`,
        id,
        payload: {
            ...payload,
        },
        ...extra,
    };
};
export const collection_deleteAction = (nodeName, extra) => (id) => {
    return {
        type: `DELETE_${toSingularAction(nodeName)}`,
        id,
        payload: {},
        ...extra,
    };
};
export const collection_setManyAction = (nodeName, extra) => (payload) => {
    return {
        type: `SET_MANY_${nodeName.toUpperCase()}`,
        payload,
        ...extra,
    };
};
export const collection_all = (nodeName, extra) => {
    return {
        get: collection_getAction(nodeName, extra),
        setAll: collection_setAllAction(nodeName, extra),
        set: collection_setAction(nodeName, extra),
        add: collection_addAction(nodeName, extra),
        patch: collection_patchAction(nodeName, extra),
        delete: collection_deleteAction(nodeName, extra),
        setMany: collection_setManyAction(nodeName, extra),
    };
};
// ============== grouped list ==============
export const groupedList_getAction = (nodeName, extra) => (payload = {}) => {
    return {
        type: `GET_${nodeName.toUpperCase()}`,
        payload,
        ...extra,
    };
};
export const groupedList_setAllAction = (nodeName, extra) => (payload) => {
    return {
        type: `SET_${nodeName.toUpperCase()}`,
        payload,
        ...extra,
    };
};
export const groupedList_addAction = (nodeName, extra) => (payload) => {
    return {
        type: `ADD_${toSingularAction(nodeName)}`,
        payload: {
            ...payload,
        },
        ...extra,
    };
};
export const groupedList_setAction = (nodeName, extra) => (id, payload) => {
    return {
        type: `SET_${toSingularAction(nodeName)}`,
        id,
        payload: {
            ...payload,
        },
        ...extra,
    };
};
export const groupedList_patchAction = (nodeName, extra) => (id, payload) => {
    return {
        type: `PATCH_${toSingularAction(nodeName)}`,
        id,
        payload: {
            ...payload,
        },
        ...extra,
    };
};
export const groupedList_deleteAction = (nodeName, extra) => (id) => {
    return {
        type: `DELETE_${toSingularAction(nodeName)}`,
        id,
        payload: {},
        ...extra,
    };
};
export const groupedList_setManyAction = (nodeName, extra) => (payload) => {
    return {
        type: `SET_MANY_${nodeName.toUpperCase()}`,
        payload,
        ...extra,
    };
};
export const groupedList_getItems = (nodeName, extra) => (id, payload) => {
    return {
        type: `GET_${nodeName.toUpperCase()}_ITEMS`,
        id,
        payload: {
            ...payload,
        },
        ...extra,
    };
};
export const groupedList_setItems = (nodeName, extra) => (id, payload) => {
    return {
        type: `SET_${nodeName.toUpperCase()}_ITEMS`,
        id,
        payload: {
            items: payload,
        },
        ...extra,
    };
};
export const groupedList_pushItem = (nodeName, extra) => (id, payload) => {
    return {
        type: `PUSH_${nodeName.toUpperCase()}_ITEM`,
        id,
        payload: {
            items: [payload],
        },
        ...extra,
    };
};
export const groupedList_popItem = (nodeName, extra) => (id) => {
    return {
        type: `POP_${nodeName.toUpperCase()}_ITEM`,
        id,
        payload: {},
        ...extra,
    };
};
export const groupedList_deleteItem = (nodeName, extra) => (id, itemId) => {
    return {
        type: `DELETE_${nodeName.toUpperCase()}_ITEM`,
        id,
        itemId,
        payload: {},
        ...extra,
    };
};
export const groupedList_patchItem = (nodeName, extra) => (id, itemId, payload) => {
    return {
        type: `PATCH_${nodeName.toUpperCase()}_ITEM`,
        id,
        itemId,
        payload: {
            ...payload,
        },
        ...extra,
    };
};
export const groupedList_clearItems = (nodeName, extra) => (id) => {
    return {
        type: `CLEAR_${nodeName.toUpperCase()}_ITEMS`,
        id,
        payload: {},
        ...extra,
    };
};
export const groupedList_pushManyItems = (nodeName, extra) => (id, payload) => {
    return {
        type: `PUSH_MANY_${nodeName.toUpperCase()}_ITEMS`,
        id,
        payload: {
            items: payload,
        },
        ...extra,
    };
};
export const groupedList_all = (nodeName, extra) => {
    return {
        get: groupedList_getAction(nodeName, extra),
        setAll: groupedList_setAllAction(nodeName, extra),
        set: groupedList_setAction(nodeName, extra),
        add: groupedList_addAction(nodeName, extra),
        patch: groupedList_patchAction(nodeName, extra),
        delete: groupedList_deleteAction(nodeName, extra),
        setMany: groupedList_setManyAction(nodeName, extra),
        getItems: groupedList_getItems(nodeName, extra),
        setItems: groupedList_setItems(nodeName, extra),
        pushItem: groupedList_pushItem(nodeName, extra),
        patchItem: groupedList_patchItem(nodeName, extra),
        deleteItem: groupedList_deleteItem(nodeName, extra),
        popItem: groupedList_popItem(nodeName, extra),
        clearItems: groupedList_clearItems(nodeName, extra),
        pushManyItems: groupedList_pushManyItems(nodeName, extra),
    };
};
// ============== from store structure ==============
export const generateActionsForStore = (storeState) => {
    let output = {};
    const keys = Object.keys(storeState);
    for (const key of keys) {
        const value = storeState[key];
        const type = nodeToType(value);
        switch (type) {
            case NodeType.SINGLE_NODE:
                output[key] = single_all(String(key), SIGNATURE);
                break;
            case NodeType.QUEUE_NODE:
                output[key] = queue_all(String(key), SIGNATURE);
                break;
            case NodeType.COLLECTION_NODE:
                output[key] = collection_all(String(key), SIGNATURE);
                break;
            case NodeType.GROUPED_LIST_NODE:
                output[key] = groupedList_all(String(key), SIGNATURE);
                break;
        }
    }
    return output;
};
