import { NodeType } from './types/types';
const isQueueNode = (value) => {
    return Array.isArray(value);
};
const isCollectionNode = (value) => {
    if (typeof value !== 'object')
        return false;
    const firstKey = Object.keys(value).pop();
    return value && value[firstKey] && value[firstKey].id === firstKey;
};
const isGroupedListNode = (value) => {
    if (typeof value !== 'object')
        return false;
    const firstKey = Object.keys(value).pop();
    const firstValue = firstKey ? value[firstKey] : null;
    const { items } = firstValue ?? {};
    return isCollectionNode(value) && Array.isArray(items);
};
const isSingleNode = (value) => {
    if (typeof value !== 'object')
        return false;
    return !value['id'];
};
export const nodeToType = (value) => {
    if (isQueueNode(value)) {
        return NodeType.QUEUE_NODE;
    }
    else if (isGroupedListNode(value)) {
        return NodeType.GROUPED_LIST_NODE;
    }
    else if (isCollectionNode(value)) {
        return NodeType.COLLECTION_NODE;
    }
    else if (isSingleNode(value)) {
        return NodeType.SINGLE_NODE;
    }
    return undefined;
};
export const analyzeStructure = (json) => {
    const memo = {};
    return Object.keys(json).reduce((output, key) => {
        if (nodeToType(json[key])) {
            output[key] = nodeToType(json[key]);
        }
        return output;
    }, memo);
};
export const clearNodes = (json, nodes) => {
    // generating a new object for immutability sake
    return Object.keys(json)
        .filter((key) => !nodes.includes(key))
        .reduce((output, key) => {
        output[key] = json[key];
        return output;
    }, {});
};
